package com.yizhi.statistics.application.commands;

import com.baomidou.mybatisplus.toolkit.CollectionUtils;
import com.yizhi.course.application.vo.CourseAccountReport;
import com.yizhi.statistics.application.entity.StatisticsAccount;
import com.yizhi.statistics.application.entity.StatisticsCourseVisibleRange;
import com.yizhi.statistics.application.util.JedisUtil;
import com.yizhi.statistics.application.util.StatisticsAccountUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.datasource.DataSourceUtils;
import org.springframework.stereotype.Component;

import javax.sql.DataSource;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * 报表任务管理类，采用线程池实现多线程执行IStatisticsMetadataTask接口的executeMetadata方法，该类主要负责内容：
 * 1、根据业务的站点或可见范围查询应参加人员信息
 * 	     查询站点或可见范围应参人员，采用了分布式处理方式，会有多个业务处理相同站点或可见范围的人员信息，
 *    如有其它业务查询相同数据，那当前业务就等待其完成，完成后获取到应参加人信息在执行自己的业务
 *    在等待处理方法：会循环检查有没有处理好的应参加人员，有就执行具体的统计业务，没有就继续等待
 * 2、将应参加人传递给IStatisticsMetadataTask接口的回调方法（executeMetadata），执行具体的业务操作
 * 
 * @author meicz
 *
 */
@Component
public abstract class StatisticsMetadataJobTrackerManage {
	
	private Logger logger = LoggerFactory.getLogger(StatisticsMetadataJobTrackerManage.class);
	
	@Autowired
	private StatisticsAccountUtil statisticsAccountUtil;
	
	@Autowired
	private JedisUtil jedisUtil;

	@Autowired
	private DataSource dataSource;
	
	//字段分割符
	private final static String FIELD_STERMINATED = "','";		
	//行分割符
	private final static String LINE_STERMINATED = "'\r\n'";
		
	private final static String FENGEFU = ",";  //tab键
	private final static String HUANHANGFU = "\r\n";  //换行
			
	private StringBuilder strBuilder = new StringBuilder();
	
	private Object objlock = new Object();

	private final static String STATISTICS_COURSE_VISIBLE_RANGE_COLUMNS = "(id,company_id,site_id,course_id,relation_id,is_virtual,type,create_time)";
	private final static String STATISTICS_COURSE_VISIBLE_RANGE_EXECUTE_SQL = "load data local infile ''  into table statistics_course_visible_range fields terminated by "
			+ FIELD_STERMINATED + "enclosed by '' escaped by 'N'" + " lines terminated by " + LINE_STERMINATED + " "
			+ STATISTICS_COURSE_VISIBLE_RANGE_COLUMNS;
	
	
	//线程池
	ExecutorService executor = Executors.newFixedThreadPool(8);
	

	
	static SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");

	/**
	 *  查询应参加人员信息，查询到人员后在执行业务回调方法执行具体的业务操作
	 * @param metadataObject
	 * @param workId
	 * @param startDate
	 * @param endDate
	 * @param taskType   0课程 1考试 2调研  3直播  4投票  5作业   6签到
	 * @param companyId
	 * @param siteId
	 * @param callbackTask  传this关键字
	 * @param newScope      全平台传1  指定范围传2   如果是指定范围（newScope=2）确保visibleIds有值，如果newScope=2 visibleIds为空，这个业务的数据会出错
	 * @throws IOException
	 */
	@SuppressWarnings({ "rawtypes", "unchecked" })
	public void executeTask(Long taskId, Object metadataObject, Long workId, Date startDate, Date endDate, int taskType,
							Long companyId, Long siteId, CourseAccountReport orgIdsAndAccountIds, IStatisticsMetadataTask callbackTask, int newScope) {
		try {
			List<StatisticsCourseVisibleRange> vs= statisticsAccountUtil.dealScopeMessage(orgIdsAndAccountIds, companyId, siteId, workId,newScope, taskType);
			if(CollectionUtils.isNotEmpty(vs)) {
				synchronized(objlock){
					switchStreamStatisticsCourseVisibleRange(vs);
				}
			}
			// 执行业务数据
			callbackTask.executeMetadata(taskId,metadataObject, startDate, endDate, companyId, siteId);
		} catch (Exception e) {
			// TODO Auto-generated catch block

			e.printStackTrace();
			callbackTask.exceptionLog(taskId, metadataObject, e);
		}




	}
	

	private void waitExecuteTask(Long taskId,Object metadataObject, Long workId, Date startDate, Date endDate, int taskType,
			Long companyId, Long siteId, CourseAccountReport orgIdsAndAccountIds,Integer scope, IStatisticsMetadataTask callbackTask, int newScope){
		String cashKey = jedisUtil.cashKey(siteId, orgIdsAndAccountIds,scope);
		List<StatisticsAccount> statisticsAccounts = null;
		try {
			while(statisticsAccounts != null){
				statisticsAccounts = jedisUtil.getValue(cashKey);
			}
			// 可见范围信息入库
			statisticsAccountUtil.dealScopeMessage(orgIdsAndAccountIds, companyId, siteId, workId,
					newScope, taskType);
			callbackTask.executeMetadata(taskId,metadataObject, startDate, endDate, companyId, siteId);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	public void switchStreamStatisticsCourseVisibleRange(List<StatisticsCourseVisibleRange> list) {
		SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		for (StatisticsCourseVisibleRange sc : list) {
			strBuilder.append(sc.getId());
			strBuilder.append(FENGEFU);
			
			strBuilder.append(sc.getCompanyId());		
			strBuilder.append(FENGEFU);
			
			strBuilder.append(sc.getSiteId());
			strBuilder.append(FENGEFU);
			
			strBuilder.append(sc.getCourseId());
			strBuilder.append(FENGEFU);
			
			strBuilder.append(sc.getRelationId());
			strBuilder.append(FENGEFU);
			 						
			strBuilder.append(sc.getIsVirtual());
			strBuilder.append(FENGEFU);
			
			strBuilder.append(sc.getType());
			strBuilder.append(FENGEFU);
			
			if (sc.getCreateTime() != null) {
				strBuilder.append(simpleDateFormat.format(sc.getCreateTime()));
			}
			strBuilder.append(FENGEFU);
			
			strBuilder.append(HUANHANGFU);
		}
	}
	
	/**
	 * 添加可见范围信息
	 * 等所有跑批完成了才执行
	 * @return
	 */
	public boolean insertBatchStatisticsCourseVisibleRange() {
		Connection conn = null;
		InputStream in = null;
		PreparedStatement preStatement = null;
		com.mysql.jdbc.PreparedStatement mysqlPreStatement = null;
		try{
			// sql编译
			conn = DataSourceUtils.getConnection(dataSource);
			preStatement = conn.prepareStatement(STATISTICS_COURSE_VISIBLE_RANGE_EXECUTE_SQL);
			mysqlPreStatement = preStatement.unwrap(com.mysql.jdbc.PreparedStatement.class);
			byte[] bytes = strBuilder.toString().getBytes();
			in = new ByteArrayInputStream(bytes);

			mysqlPreStatement.setLocalInfileInputStream(in);
			// 执行
			preStatement.executeQuery();
			
			//大对象手动显示的设置为空保证GC时被释放
			bytes = null;
			//清空之前的内容
			strBuilder.delete(0, strBuilder.length());
		} catch(Exception e){
			e.printStackTrace();
			logger.info("添加可见范围数据到statistics_course_visible_range失败");
			return false;
		} finally {
			//大对象手动显示的设置为空保证GC时被释放
			if(in != null){
				in = null;
			}
			
			try{
				//关闭数据库连接
				if (mysqlPreStatement != null) {
					mysqlPreStatement.close();
				}
				if (preStatement != null) {
					preStatement.close();
				}
				if (conn != null) {
					DataSourceUtils.releaseConnection(conn, dataSource);
				}
			} catch(Exception e){
				e.printStackTrace();
				return false;
			}
		}

		return true;
	}
}
