package com.yizhi.statistics.application.controller.forum;

import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.forum.application.feign.remote.RemotePostsClient;
import com.yizhi.forum.application.feign.student.StudentPostsFavoriteClient;
import com.yizhi.forum.application.vo.domain.PostsVo;
import com.yizhi.statistics.application.entity.StatisticsForum;
import com.yizhi.statistics.application.service.IStatisticsForumService;
import com.yizhi.system.application.system.remote.AccountClient;
import com.yizhi.system.application.vo.AccountVO;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.*;

/**
 * 跑批任务
 */
@Component
public class StatisticsForumAbs extends AbstractDefaultTask<String, Map<String, Object>> {

    @Autowired
    private IdGenerator idGenerator;

    @Autowired
    private IStatisticsForumService statisticsForumService;
    @Autowired
    private AccountClient accountClient;
    @Autowired
    private RemotePostsClient remotePostsClient;
    @Autowired
    private StudentPostsFavoriteClient postsFavoriteClient;


    final SimpleDateFormat sim = new SimpleDateFormat("yyyy-MM-dd");

    private static final Logger LOGGER = LoggerFactory.getLogger(StatisticsForumAbs.class);


    @Override
    protected String execute(Map<String, Object> map) {
        Long taskId = idGenerator.generate();
        Long taskStartTime = System.currentTimeMillis();
        /**
         * 走异步任务
         */
        Date curDateWork = new Date();

        TaskContext taskContext = new TaskContext(taskId, sim.format(curDateWork) + " 号0:10论坛按帖子统计报表分析前一天的数据", 999L, curDateWork);
        working(taskContext);

        String result = "";
        //先获取所有已上架的帖子
        List<PostsVo> allPosts = null;
        try {
            allPosts = remotePostsClient.getAllPosts();
        } catch (Exception e) {
            result = "***** 论坛按帖子统计报表,获取帖子数据时出现异常 *****";
            taskDetail(taskId, result);
            LOGGER.info(result);
            return "error";
        }
        if (CollectionUtils.isEmpty(allPosts)) {
            result = "***** 论坛按帖子统计报表,帖子数量为0 正常退出 *****";
            taskDetail(taskId, result);
            LOGGER.info(result);
            return "error";
        }
        //先得清空statistics_forum表数据
        try {
            statisticsForumService.deleteAllData();
            result = "***** 论坛统计插入数据开始前，先删除数据*****";
            taskDetail(taskId, result);
            LOGGER.info(result);
        } catch (Exception e) {
            result = "***** 论坛统计插入数据开始前,删除数据，出现异常*****";
            taskDetail(taskId, result);
            LOGGER.info(result);
            e.printStackTrace();
        }

        Set<Long> accountIds = new HashSet<>(allPosts.size());
        Set<Long> postsIds = new HashSet<>(allPosts.size());
        for (PostsVo a : allPosts) {
            accountIds.add(a.getAuthor());
            postsIds.add(a.getId());
        }
        //获取用户信息
        Map<Long, AccountVO> accountVOMap = queryAccountByIds(new ArrayList<>(accountIds));
        //获取帖子点赞数
        Map<Long, Integer> favoriteMap = null;
        try {
            favoriteMap = postsFavoriteClient.hasFavoriteByPostsIds(new ArrayList<>(postsIds));
        } catch (Exception e) {
            result = "***** 论坛按帖子统计报表,获取帖子收藏数据时出现异常 *****";
            taskDetail(taskId, result);
            LOGGER.info(result);
        }

        result = "***** 论坛按帖子统计报表 开始插入数据*****";
        taskDetail(taskId, result);
        LOGGER.info(result);
        List<StatisticsForum> statisticsForums = new ArrayList<>(allPosts.size());
        for (PostsVo a : allPosts) {
            statisticsForums.add(buildRecord(a, accountVOMap, favoriteMap));
        }

        try {
            statisticsForumService.insertBatch(statisticsForums);
        } catch (Exception e) {
            result = "***** 论坛按帖子统计报表 插入数据 出现异常*****";
            taskDetail(taskId, result);
            LOGGER.info(result);
            e.printStackTrace();
        }
        result = "***** 论坛统计 插入数据完成*****";
        taskDetail(taskId, result);
        LOGGER.info(result);
        Long taskEndTime = System.currentTimeMillis();
        Long t = (taskEndTime - taskStartTime);
        Integer count = allPosts.size();

        success(taskContext, "执行成功,耗时:" + t + "s", "");

        taskDetail(taskId, "总数量:" + count + "。处理时间ms:" + t);
        LOGGER.info("***** 数据入库结束 ,数量:" + count + "。处理时间ms:" + t + " *****");
        return "success";

    }

    public StatisticsForum buildRecord(PostsVo a, Map<Long, AccountVO> accountVOMap, Map<Long, Integer> favoriteMap) {
        StatisticsForum statisticsForum = new StatisticsForum();
        statisticsForum.setId(idGenerator.generate());
        statisticsForum.setPostsId(a.getId());
        BeanUtils.copyProperties(a, statisticsForum);
        statisticsForum.setAccountId(a.getAuthor());
        statisticsForum.setCommentNum(a.getCommentNum() + a.getReplyNum());
        statisticsForum.setPostsCreateTime(a.getCreateTime());
        statisticsForum.setCreateTime(new Date());
        if (accountVOMap != null) {
            AccountVO accountVO = accountVOMap.get(a.getAuthor());
            statisticsForum.setAccountName(accountVO == null ? "" : accountVO.getName());
            statisticsForum.setFullName(accountVO == null ? "" : accountVO.getFullName());
        }
        //收藏数
        if (favoriteMap != null) {
            statisticsForum.setFavoriteNum(favoriteMap.get(a.getId()));
        }
        return statisticsForum;
    }

    /**
     * 返回《accountId,account》的map
     *
     * @param accountIds
     * @return
     */
    public Map<Long, AccountVO> queryAccountByIds(List<Long> accountIds) {
        if (!CollectionUtils.isEmpty(accountIds)) {
            Map<Long, AccountVO> map = new HashMap<>(accountIds.size());
            List<AccountVO> accountVOS = accountClient.findByIds(accountIds);
            //组装map 以免双重循环
            if (!org.springframework.util.CollectionUtils.isEmpty(accountVOS)) {
                accountVOS.forEach(a -> {
                    if (!map.containsKey(a.getId())) {
                        map.put(a.getId(), a);
                    }
                });
            }
            return map;
        }
        return null;
    }
}
