package com.yizhi.statistics.application.util;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.toolkit.CollectionUtils;
import com.yizhi.core.application.cache.RedisCache;
import com.yizhi.course.application.vo.CourseAccountReport;
import com.yizhi.statistics.application.entity.StatisticsAccount;
import com.yizhi.statistics.application.mapper.StatisticsAccountMapper;
import com.yizhi.statistics.application.mapper.StatisticsCourseRecordeMapper;
import com.yizhi.statistics.application.scope.RedisKey;
import com.yizhi.statistics.application.work.CourseService;
import com.yizhi.system.application.vo.ReportAccountRespVO;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.datasource.DataSourceUtils;
import org.springframework.stereotype.Component;

import javax.sql.DataSource;
import java.io.IOException;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;

/**
 * 处理人的信息和redis的关系
 * 
 * @author wangfeida
 *
 */
@Component
public class JedisUtil {

	private Logger logger = LoggerFactory.getLogger(JedisUtil.class);
	private static final int expireTime = 3600 * 26;


	@Autowired
	private WorkUtil workUtil;

	@Autowired
	private SystemAccountUtil systemAccountUtil;

	@Autowired
	private RedisCache redisCache;

	@Autowired
	private StatisticsAccountMapper statisticsAccountMapper;
	
	@Autowired
	private StatisticsCourseRecordeMapper statisticsCourseRecordeMapper;

	static SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
	
	 @Autowired
	 private CourseService courseService;

	/**
	 * 尝试获取分布式锁
	 * 
	 * @return 是否获取成功
	 * @throws IOException
	 * @throws ClassNotFoundException
	 */

	public boolean lock(String cachKey) throws ClassNotFoundException, IOException {
		String key = cachKey + "_lock_statistics";
		String value = "xxxx";
		Boolean result = redisCache.setIfAbsent(key, value);// 0 已经存在 1原始不存在
		if (result) {// 如果不存在，存储完，给设置过期时间
			redisCache.expire(key, expireTime);
			return true;
		}
		// 如果已经存在，证明有别的服务给设置过
		return false;
	}

	/**
	 * 锁释放
	 * 
	 * @param cachKey
	 */
	public void dealLock(String cachKey) {
		String key = cachKey + "_lock_statistics";
		redisCache.delete(key);
	}

	/**
	 * 将在system查到的值赋值给redis
	 * 
	 * @param list    查到的人的信息
	 * @return
	 * @throws IOException
	 * @throws ClassNotFoundException
	 */
	public boolean setValue(String cachKey, List<StatisticsAccount> list) throws IOException, ClassNotFoundException {
		String key = cachKey + "_value_";
		// JSONArray jsonString = JSONArray.parseArray(JSON.toJSONString(list));
		String jsonString = JSON.toJSONString(list);
		Boolean result = redisCache.set(key, jsonString);
		if (result) { // 如果但是不存在，存储完，给设置过期时间
			redisCache.expire(key, expireTime);// 过期时间1小时 1成功 0不存在
			return true;
		}
		// 如果已经存在，证明设置失败
		return false;
	}

	/**
	 * 从redis取到值,取的是system查出来的人员信息
	 * 
	 * @param cachKey
	 * @return
	 * @throws IOException
	 */
	public List<StatisticsAccount> getValue(String cachKey) throws IOException {
		List<StatisticsAccount> list = null;
		String key = cachKey + "_value_";
		Object value = redisCache.get(key);
		if (value != null) {
			JSONArray jsonArray = JSONArray.parseArray(value.toString());
			list = JSONObject.parseArray(jsonArray.toJSONString(), StatisticsAccount.class);
		}
		return list;
	}

	/**
	 * 
	 * @param companyId               业务所在公司id
	 * @param siteId                  业务所在站点id
	 * @return
	 * @throws ClassNotFoundException
	 * @throws IOException
	 */
	public List<StatisticsAccount> getStatisticsAccount(Long companyId, Long siteId,
														CourseAccountReport orgIdsAndAccountIds, String cachKey, Integer scope)
			throws ClassNotFoundException, IOException {

		// 获取缓存中的值
		List<StatisticsAccount> statisticsAccounts = getValue(cachKey);
		if (statisticsAccounts != null) {
			return statisticsAccounts;
		}

		// 获取不到值，执行加锁查询，防止其他业务处理相同站点或可见范围人员信息
		boolean isLock = lock(cachKey);
		if (isLock) {// 自己锁定自己处理
			List<ReportAccountRespVO> listSystemAccount = systemAccountUtil.getAccountMessageFromSystem(companyId,
					siteId, orgIdsAndAccountIds, scope);
			statisticsAccounts = listSystemSwitch(listSystemAccount, companyId, siteId);
			// 放在redis
			setValue(cachKey, statisticsAccounts);

			// 释放lock方法生成的锁cachKey + "_lock_statistics"
			// dealLock(cachKey);
		}
		if (!isLock) {// 别的服务已经处理过
			statisticsAccounts = getValue(cachKey);
			if (statisticsAccounts != null) {
				return statisticsAccounts;
			}
		}
		// 如果返回有值，证明处理过 如果为null 别的服务正在处理 如果不为null list.size()=0 没有查到值
		return statisticsAccounts;
	}

	public String cashKey(Long siteId, CourseAccountReport orgIdsAndAccountIds, Integer scope) {
		// 统一cashKey
		String cachKey = siteId.toString();
		// 如果外边没有进行判断orgIdsAndAccountIds以及orgIds和accuntIds 就按照全平台可见处理
		if (scope == 2) {
			if (orgIdsAndAccountIds != null) {
				List<Long> ids = new ArrayList<Long>();
				if (CollectionUtils.isNotEmpty(orgIdsAndAccountIds.getOrgIds())) {
					ids.addAll(orgIdsAndAccountIds.getOrgIds());
				}
				if (CollectionUtils.isNotEmpty(orgIdsAndAccountIds.getAccountIds())) {
					ids.addAll(orgIdsAndAccountIds.getAccountIds());
				}
				Collections.sort(ids);
				cachKey = orgIdsAndAccountIds.toString();
			}
		}
		return cachKey;
	}

	/**
	 * 将system服务得到的集合对象转换为报表服务得到的对象
	 * 
	 * @param listSystem   system查询到的人的集合
	 * @param companyId    当前业务的companyId
	 * @param siteId       当前业务的siteId
	 * @param relationCode 可见范围
	 * @return
	 */
	private List<StatisticsAccount> listSystemSwitch(List<ReportAccountRespVO> listSystem, Long companyId,
			Long siteId) {
		List<StatisticsAccount> listReactTo = new ArrayList<StatisticsAccount>();
		StatisticsAccount statisticsAccount = null;
		for (ReportAccountRespVO system : listSystem) {
			statisticsAccount = new StatisticsAccount();
			statisticsAccount.setCompanyId(companyId);
			statisticsAccount.setSiteId(siteId);
			statisticsAccount.setOrgId(system.getOrgId());
			statisticsAccount.setOrgCode(system.getOrgCode());
			statisticsAccount.setOrgName(system.getOrgName());
			String parentOrgNames = "";
			if (CollectionUtils.isNotEmpty(system.getParentOrgNames())) {
				for (int i = 0; i < system.getParentOrgNames().size(); i++) {
					parentOrgNames = parentOrgNames + "/" + system.getParentOrgNames().get(i);
				}
			}
			statisticsAccount.setOrgNames(parentOrgNames);
			statisticsAccount.setAccountId(system.getUserId());
			statisticsAccount.setAccountName(system.getUserName());
			statisticsAccount.setFullName(system.getUserFullName());
			statisticsAccount.setWorkNum(system.getWorkNum());
			statisticsAccount.setAccountState(system.getStatus());
			listReactTo.add(statisticsAccount);
		}
		return listReactTo;
	}

	/**
	 * 查询全平台可见的课程的可参加人数
	 * 
	 * @param siteId
	 * @return
	 */
	public Integer getSiteIdJoinNum(Long siteId) {
		Object siteIdJoinNumObject = redisCache.get("statistics_site_joinNum_" + siteId);
		if (siteIdJoinNumObject != null) {
			Integer siteIdJoinNum = (Integer) siteIdJoinNumObject;
			return siteIdJoinNum;
		} else {
			Integer siteIdJoinNum = statisticsAccountMapper.siteIdJoinNum(siteId);
			if (siteIdJoinNum != null) {
				redisCache.set("statistics_site_joinNum_" + siteId, siteIdJoinNum);
			}
			return siteIdJoinNum;
		}
	}

	/**
	 * 对于超级管理员讲，每个业务的课参加人数
	 * 
	 * @param workId
	 * @param totalAskAccount
	 */
	@Deprecated
	public void setTotalAskAccount(Long workId, Integer totalAskAccount) {
		redisCache.set("statistics_totalaskjoinnum_byworkid_" + workId, totalAskAccount);
	}

	@Deprecated
	public Integer getTotalAskAccountByWorkId(Long workId) {
		Object getTotalAskAccountByWorkId = redisCache.get("statistics_totalaskjoinnum_byworkid_" + workId);
		Integer totalAskAccountByWorkId = 0;
		if (getTotalAskAccountByWorkId != null) {
			totalAskAccountByWorkId = Integer.valueOf(getTotalAskAccountByWorkId.toString());
		}
		return totalAskAccountByWorkId;
	}

	// 不管哪个业务都是可以用这个
	public void setAskNumByAdminIdAndWorkId(Long adminId, Long siteId, Long workId, Long askNum) {
		if (askNum == null) {
			askNum = 0L;
		}
		redisCache.set("statisticsTree:totalAskJoinNnum:byAdminIdWorkId:" + adminId + "_" + siteId + "_" + workId,
				askNum.toString());
	}

	public Integer getAskNumByAdminIdAndWorkId(Long adminId, Long siteId, Boolean isAdmin, Long workId) {
		Object getTotalAskAccountByWorkId = redisCache
				.get("statisticsTree:totalAskJoinNnum:byAdminIdWorkId:" + adminId + "_" + siteId + "_" + workId);
		Integer totalAskAccountByWorkId = 0;
		if (getTotalAskAccountByWorkId != null) {
			totalAskAccountByWorkId = Integer.valueOf(getTotalAskAccountByWorkId.toString());
		}
		return totalAskAccountByWorkId;
	}
	
	
	public void deleteLaZiDate(Long adminId,Long siteId,Long workId) {
		String key="statisticsTree:totalAskJoinNnum:byAdminIdWorkId:" + adminId + "_" + siteId + "_" + workId;
		redisCache.delete(key);
	}

	public void setWorkIsSuccess(Integer workType, Date curDate) {
		//每个业务跑成功都会把成功消息放在redis
		String dateString = sdf.format(curDate);
		String key = "statistics:" + workType + ":success:" + dateString;
		redisCache.set(key, "success");
		redisCache.expire(key, expireTime);

		checkAskJoinNumIsSetSuccess(curDate, workType);

	}

	public void deleteWorkIsSuccess(Date date) {
		String courseKey =RedisKey.getRedisKeyByTypeAndDate(0, date);
		redisCache.delete(courseKey);
		
		String examKey =RedisKey.getRedisKeyByTypeAndDate(1, date);
		redisCache.delete(examKey);
		
		String researchKey =RedisKey.getRedisKeyByTypeAndDate(2, date);
		redisCache.delete(researchKey);
		
		String signKey =RedisKey.getRedisKeyByTypeAndDate(6, date);
		redisCache.delete(signKey);
		
		String trainingKey =RedisKey.getRedisKeyByTypeAndDate(7, date);
		redisCache.delete(trainingKey);
		
		String systemKey = RedisKey.getRedisKeyByTypeAndDate(-2, date);
		redisCache.delete(systemKey);
		
		String roleKey = RedisKey.getRedisKeyByTypeAndDate(-1, date);
		redisCache.delete(roleKey);
		
		String threadKey =RedisKey.getRedisKeyThread(date);
		redisCache.delete(threadKey);
	}

	public void checkAskJoinNumIsSetSuccess(Date date, Integer type) {
		String courseKey = RedisKey.getRedisKeyByTypeAndDate(0, date);
		String courseIsSuccess = (String) redisCache.get(courseKey);
		
		String examKey = RedisKey.getRedisKeyByTypeAndDate(1, date);
		String examIsSuccess = (String) redisCache.get(examKey);
		
		String researchKey =RedisKey.getRedisKeyByTypeAndDate(2, date);
		String researchIsSuccess = (String) redisCache.get(researchKey);
		
		String trainingKey =RedisKey.getRedisKeyByTypeAndDate(7, date);
		String trainingIsSuccess = (String) redisCache.get(trainingKey);
		
		String systemKey = RedisKey.getRedisKeyByTypeAndDate(-2, date);
		String systemSuccess = (String) redisCache.get(systemKey);
		
		String roleKey =RedisKey.getRedisKeyByTypeAndDate(-1, date);
		String roleSuccess = (String) redisCache.get(roleKey);
				
		String signKey = RedisKey.getRedisKeyByTypeAndDate(6, date);
		String signSuccess = (String) redisCache.get(signKey);
		
		if("success".equals(courseIsSuccess)&& "success".equals(systemSuccess)) {
			//更新课程的记录的组织信息
			try {
				//课程的
				String sqlCourseRecorde="UPDATE statistics_course_recorde a,statistics_account b SET a.org_id=b.org_id WHERE a.account_id=b.account_id AND a.org_id IS NULL;";
				String sqlCourseMessage="UPDATE statistics_course_message a,statistics_account b SET a.org_id=b.org_id WHERE a.account_id=b.account_id AND a.org_id IS NULL;";
				testProce(sqlCourseRecorde);
				testProce(sqlCourseMessage);
			} catch (SQLException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
				
		if("success".equals(systemSuccess)&&"success".equals(trainingIsSuccess)) {
			try {
				//更新培训项目的组织信息
				String sqlTrainingStudentActivity="UPDATE statistics_training_student_activity_record a,statistics_account b SET a.org_id=b.org_id WHERE a.account_id=b.account_id AND a.org_id IS NULL;";
				testProce(sqlTrainingStudentActivity);
			} catch (SQLException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		

		logger.info("type:" + type +  " courseKey:" + courseIsSuccess + " _examKey:" + examIsSuccess + " _researchKey:" + researchIsSuccess
				+ " _systemKey:" + systemSuccess + " _roleKey:" + roleSuccess+ " _trainingKey:" + trainingIsSuccess + " _signKey:" + signSuccess);
		if ("success".equals(courseIsSuccess) && "success".equals(examIsSuccess) && "success".equals(researchIsSuccess) &&
			"success".equals(trainingIsSuccess) && "success".equals(systemSuccess) && "success".equals(roleSuccess) && 
			"success".equals(signSuccess)) {
			// 如果各个模块跑完开始跑redis
			String threadKey=RedisKey.getRedisKeyThread(date);
			boolean isNoRunning = redisCache.setIfAbsent(threadKey, "running");
			if (isNoRunning) {
				logger.info("最后跑完的是type:" + type + " 可见范围入库");
				//添加可见范围
				courseService.insertBatchStatisticsCourseVisibleRange();
				
				//设置可参加人数缓存
				workUtil.setAskJoinNumByWorkIdAndNotAdminBySql();
				
				// 删除缓存
				deleteWorkIsSuccess(date);
			}

		}
	}
	
	
	
	@Autowired
	private DataSource dataSource;	
	
	public void testProce(String sql) throws SQLException {
		Connection conn = null;
		Statement statement = null;
	
		try{
			// sql编译
			conn = DataSourceUtils.getConnection(dataSource);
			statement=conn.createStatement();
			
			statement.executeUpdate(sql);
		} catch(Exception e){
			e.printStackTrace();
		} finally {
			if(statement!=null) {
				statement.close();
			}
			if (conn != null) {
				DataSourceUtils.releaseConnection(conn, dataSource);
			}
		}
	}
}
